@@ -314,10 +314,6 @@ Data: %3
         <source>Changed revision to:</source>
         <translation>Změněna revize:</translation>
     </message>
-    <message>
-        <source>Saving as</source>
-        <translation>Ukládám jako</translation>
-    </message>
     <message>
         <source>Operators</source>
         <translation>Operátory</translation>
@@ -1055,6 +1051,54 @@ Data: %3
         <source>xml files (*.xml)</source>
         <translation>xml soubory (*.xml)</translation>
     </message>
+    <message>
+        <source>Save as text file</source>
+        <translation>Uložit jako textový soubor</translation>
+    </message>
+    <message>
+        <source>Text saved to %1</source>
+        <translation>Text uložen do %1</translation>
+    </message>
+    <message>
+        <source>Font face</source>
+        <translation>Písmo</translation>
+    </message>
+    <message>
+        <source>Font size (in points)</source>
+        <translation>Velikost písma (v bodech)</translation>
+    </message>
+    <message>
+        <source>Line width (in points)</source>
+        <translation>Šířka čáry (v bodech)</translation>
+    </message>
+    <message>
+        <source>Use &apos;%1 --help&apos; to see list of commandline options and their parameters</source>
+        <translation>Použijte &apos;%1 --help&apos; k zobrazení seznamu přepínačů na příkazové řádce a jejich parametrů</translation>
+    </message>
+    <message>
+        <source>Saved as</source>
+        <translation>Uloženo jako</translation>
+    </message>
+    <message>
+        <source>Unable to open file: %1</source>
+        <translation>Nelze otevřít soubor: %1</translation>
+    </message>
+    <message>
+        <source>Not implemented: %1</source>
+        <translation>Nepodporováno: %1</translation>
+    </message>
+    <message>
+        <source>Unknown error</source>
+        <translation>Neznámá chyba</translation>
+    </message>
+    <message>
+        <source>Apply from page</source>
+        <translation>Aplikovat od strany</translation>
+    </message>
+    <message>
+        <source>How many pages</source>
+        <translation>Kolik stran</translation>
+    </message>
 </context>
 <context>
     <name>annotation_type</name>
@@ -1165,18 +1209,6 @@ Data: %3
 </context>
 <context>
     <name>gui::AboutWindow</name>
-    <message>
-        <source>PDF editor for unix systems</source>
-        <translation>PDF editor pro unix systémy</translation>
-    </message>
-    <message>
-        <source>Project leader</source>
-        <translation>Vedoucí projektu</translation>
-    </message>
-    <message>
-        <source>Authors</source>
-        <translation>Autoři</translation>
-    </message>
     <message>
         <source>About program</source>
         <translation>O programu</translation>
@@ -1193,6 +1225,22 @@ Data: %3
         <source>Homepage</source>
         <translation>Stránka</translation>
     </message>
+    <message>
+        <source>Free program for PDF document manipulation</source>
+        <translation>Svobodný program pro manipulaci s PDF dokumenty</translation>
+    </message>
+    <message>
+        <source>Project page</source>
+        <translation>Stránka projektu</translation>
+    </message>
+    <message>
+        <source>Using Qt %1</source>
+        <translation>Použito Qt %1</translation>
+    </message>
+    <message>
+        <source>, QSA %1</source>
+        <translation>, QSA %1</translation>
+    </message>
 </context>
 <context>
     <name>gui::AddItemDialog</name>
@@ -1447,6 +1495,14 @@ Režim zvýraznění</translation>
         <source>Choose color</source>
         <translation>Vyberte barvu</translation>
     </message>
+    <message>
+        <source>Foreground</source>
+        <translation>Popředí</translation>
+    </message>
+    <message>
+        <source>Background</source>
+        <translation>Pozadí</translation>
+    </message>
 </context>
 <context>
     <name>gui::HelpWindow</name>
@@ -1701,6 +1757,22 @@ Režim zvýraznění</translation>
         <source>Length units</source>
         <translation>Jednotky délky</translation>
     </message>
+    <message>
+        <source>Page view</source>
+        <translation>Náhled stránky</translation>
+    </message>
+    <message>
+        <source>Command Line</source>
+        <translation>Příkazová řádka</translation>
+    </message>
+    <message>
+        <source>Initial focus</source>
+        <translation>Počáteční výběr</translation>
+    </message>
+    <message>
+        <source>This will specify which part of editor window will receive keyboard focus when new window is opened</source>
+        <translation>Toto určuje, která část editoru dostane fokus z klávesnice při otevření nového okna</translation>
+    </message>
 </context>
 <context>
     <name>gui::PageSpace</name>
@@ -1712,10 +1784,6 @@ Režim zvýraznění</translation>
         <source>%1 of %2</source>
         <translation>%1 z %2</translation>
     </message>
-    <message>
-        <source>Resizing zone must be positive integer or null !</source>
-        <translation>Zóna pro změnu velikosti musí být kladné celé číslo nebo null !</translation>
-    </message>
 </context>
 <context>
     <name>gui::PdfEditWindow</name>
@@ -1755,10 +1823,6 @@ Režim zvýraznění</translation>
         <source>Unknown error occured while saving document</source>
         <translation>Neznámá chyba při ukládání dokumentu</translation>
     </message>
-    <message>
-        <source>Error occured while saving copy</source>
-        <translation>Při ukládání kopie došlo k chybě</translation>
-    </message>
     <message>
         <source>Linearized PDF</source>
         <translation>Linearizované PDF</translation>
@@ -1779,6 +1843,18 @@ Režim zvýraznění</translation>
         <source>Selected annotations</source>
         <translation>Vybrané anotace</translation>
     </message>
+    <message>
+        <source>Saving not implemented:
+%1</source>
+        <translation>Ukládání není implementováno:
+%1</translation>
+    </message>
+    <message>
+        <source>Error occured while saving copy:
+%1</source>
+        <translation>Při ukládání kopie došlo k chybě:
+%1</translation>
+    </message>
 </context>
 <context>
     <name>gui::Property</name>
@@ -1877,6 +1953,12 @@ Režim zvýraznění</translation>
         <source>Unknown error occured while saving document</source>
         <translation>Neznámá chyba při ukládání dokumentu</translation>
     </message>
+    <message>
+        <source>Unable to save:
+%1</source>
+        <translation>Nelze uložit:
+%1</translation>
+    </message>
 </context>
 <context>
     <name>gui::RefPropertyDialog</name>
@@ -1969,11 +2051,6 @@ Režim zvýraznění</translation>
         <comment>PageToolbar</comment>
         <translation>Toolbar stránky</translation>
     </message>
-    <message>
-        <source>Reload Page</source>
-        <comment>reloadpage</comment>
-        <translation>Obnovit stránku</translation>
-    </message>
     <message>
         <source>Add object</source>
         <comment>treeAdditem</comment>
@@ -2272,7 +2349,7 @@ Režim zvýraznění</translation>
     <message>
         <source>TreeView Toolbar</source>
         <comment>TreeViewToolbar</comment>
-        <translation>Toolbar stromuobjektů</translation>
+        <translation>Toolbar stromu objektů</translation>
     </message>
     <message>
         <source>Dynamic Toolbar</source>
@@ -2289,6 +2366,21 @@ Režim zvýraznění</translation>
         <comment>drawarrowe</comment>
         <translation>Nakreslit šipku</translation>
     </message>
+    <message>
+        <source>PageRider Toolbar</source>
+        <comment>PageRiderToolbar</comment>
+        <translation>Toolbar navigace stránek</translation>
+    </message>
+    <message>
+        <source>Save as text...</source>
+        <comment>fsaveastext</comment>
+        <translation>Uložit jako text...</translation>
+    </message>
+    <message>
+        <source>Reload Page (show changes)</source>
+        <comment>reloadpage</comment>
+        <translation>Obnovit stránku (ukázat změny)</translation>
+    </message>
 </context>
 <context>
     <name>gui::TreeItemPdf</name>
