@@ -316,7 +316,7 @@ Datos: %3
     </message>
     <message>
         <source>Saving as</source>
-        <translation>Guardando como</translation>
+        <translation type="obsolete">Guardando como</translation>
     </message>
     <message>
         <source>Operators</source>
@@ -1055,6 +1055,54 @@ Datos: %3
         <source>xml files (*.xml)</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Save as text file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Text saved to %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Font face</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Font size (in points)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Line width (in points)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Use &apos;%1 --help&apos; to see list of commandline options and their parameters</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Saved as</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unable to open file: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Not implemented: %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unknown error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Apply from page</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>How many pages</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>annotation_type</name>
@@ -1167,15 +1215,15 @@ Datos: %3
     <name>gui::AboutWindow</name>
     <message>
         <source>PDF editor for unix systems</source>
-        <translation>PDF editor pro para sistemas unix</translation>
+        <translation type="obsolete">PDF editor pro para sistemas unix</translation>
     </message>
     <message>
         <source>Project leader</source>
-        <translation>Líder del proyecto</translation>
+        <translation type="obsolete">Líder del proyecto</translation>
     </message>
     <message>
         <source>Authors</source>
-        <translation>Autores</translation>
+        <translation type="obsolete">Autores</translation>
     </message>
     <message>
         <source>About program</source>
@@ -1193,6 +1241,22 @@ Datos: %3
         <source>Homepage</source>
         <translation>Homepage</translation>
     </message>
+    <message>
+        <source>Free program for PDF document manipulation</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Project page</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Using Qt %1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>, QSA %1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::AddItemDialog</name>
@@ -1446,6 +1510,14 @@ Datos: %3
         <source>Choose color</source>
         <translation>Elegir color</translation>
     </message>
+    <message>
+        <source>Foreground</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Background</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::HelpWindow</name>
@@ -1700,6 +1772,22 @@ Datos: %3
         <source>Length units</source>
         <translation>Longitud de unidades</translation>
     </message>
+    <message>
+        <source>Page view</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Command Line</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Initial focus</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>This will specify which part of editor window will receive keyboard focus when new window is opened</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::PageSpace</name>
@@ -1713,7 +1801,7 @@ Datos: %3
     </message>
     <message>
         <source>Resizing zone must be positive integer or null !</source>
-        <translation>La zona a redimensionar debe ser entero positivo o nulo !</translation>
+        <translation type="obsolete">La zona a redimensionar debe ser entero positivo o nulo !</translation>
     </message>
 </context>
 <context>
@@ -1756,7 +1844,7 @@ Datos: %3
     </message>
     <message>
         <source>Error occured while saving copy</source>
-        <translation>Error ocurrido mientras se guardaba una copia</translation>
+        <translation type="obsolete">Error ocurrido mientras se guardaba una copia</translation>
     </message>
     <message>
         <source>Linearized PDF</source>
@@ -1778,6 +1866,16 @@ Datos: %3
         <source>Selected annotations</source>
         <translation>Anotaciones seleccionadas</translation>
     </message>
+    <message>
+        <source>Saving not implemented:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Error occured while saving copy:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::Property</name>
@@ -1876,6 +1974,11 @@ Datos: %3
         <source>Unknown error occured while saving document</source>
         <translation>Error desconocido ocurrido mientras se guardaba el documento</translation>
     </message>
+    <message>
+        <source>Unable to save:
+%1</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::RefPropertyDialog</name>
@@ -1971,7 +2074,7 @@ Datos: %3
     <message>
         <source>Reload Page</source>
         <comment>reloadpage</comment>
-        <translation>&amp;Recargar página</translation>
+        <translation type="obsolete">&amp;Recargar página</translation>
     </message>
     <message>
         <source>Add object</source>
@@ -2288,6 +2391,21 @@ Datos: %3
         <comment>drawarrowe</comment>
         <translation>Dibujar flecha</translation>
     </message>
+    <message>
+        <source>PageRider Toolbar</source>
+        <comment>PageRiderToolbar</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Save as text...</source>
+        <comment>fsaveastext</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Reload Page (show changes)</source>
+        <comment>reloadpage</comment>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>gui::TreeItemPdf</name>
