@@ -1,12 +1,26 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
+/*
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
+ */
 /** @file
  ToolFactory class, factory that will create special toobar items
  @author Martin Petricek
@@ -49,7 +63,7 @@ QWidget *ToolFactory::specialItem(ToolBar *tb,const QString &item,QMainWindow *m
  if (itemName=="_revision_tool") {
   //Add RevisionTool to toolbar and return
   RevisionTool *tool =new RevisionTool(tb,"revision");
-  QObject::connect(main,SIGNAL(documentChanged(pdfobjects::CPdf*)),tool,SLOT(setDocument(pdfobjects::CPdf*)));
+  QObject::connect(main,SIGNAL(documentChanged(boost::shared_ptr<pdfobjects::CPdf>)),tool,SLOT(setDocument(boost::shared_ptr<pdfobjects::CPdf>)));
   QObject::connect(main,SIGNAL(revisionChanged(int)),tool,SLOT(updateRevision(int)));
   QObject::connect(tool,SIGNAL(revisionChanged(int)),main,SLOT(changeRevision(int)));
   tool->show();
