@@ -1,12 +1,26 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
+/*
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
+ */
 /** @file
  TreeItemRef - class holding one CRef object in tree, descendant of TreeItem
  @author Martin Petricek
@@ -17,8 +31,8 @@
 #include "treedata.h"
 #include "pdfutil.h"
 #include "util.h"
-#include <cobject.h>
-#include <cpdf.h>
+#include <kernel/cobject.h>
+#include <kernel/cpdf.h>
 
 namespace gui {
 
@@ -67,7 +81,7 @@ void TreeItemRef::addData() {
  setExpandable(true);
 }
 
-/** 
+/**
  Return reference target in text form.
  @return Refence target
 */
@@ -116,7 +130,7 @@ void TreeItemRef::setOpen(bool open) {
    }
    //Reload as "complete and opened item"
    complete=true;
-   reload();  
+   reload();
   }
  }
  //Call original method
@@ -158,7 +172,7 @@ void TreeItemRef::reloadSelf() {
 TreeItemAbstract* TreeItemRef::createChild(const QString &name,__attribute__((unused)) ChildType typ,Q_ListViewItem *after/*=NULL*/) {
  assert(name=="Target");
  QString s;
- CPdf* pdf=obj->getPdf();
+ boost::shared_ptr<CPdf> pdf=obj->getPdf().lock();
  if (!pdf) return NULL; //No document opened -> cannot parse references
                    //Should happen only while testing
  CRef* cref=dynamic_cast<CRef*>(obj.get());
@@ -171,11 +185,10 @@ TreeItemAbstract* TreeItemRef::createChild(const QString &name,__attribute__((un
 
 //See TreeItemAbstract for description of this virtual method
 bool TreeItemRef::validChild(__attribute__((unused)) const QString &name,Q_ListViewItem *oldChild) {
- CPdf* pdf=obj->getPdf();
+ boost::shared_ptr<CPdf> pdf=obj->getPdf().lock();
  if (!pdf) return false; //No document opened -> cannot parse references
- CRef* cref=dynamic_cast<CRef*>(obj.get());
  IndiRef ref;
- cref->getValue(ref);
+ IProperty::getSmartCObjectPtr<CRef>(obj)->getValue(ref);
  boost::shared_ptr<IProperty> rp=pdf->getIndirectProperty(ref);
  TreeItem* it=dynamic_cast<TreeItem*>(oldChild);
  if (!it) return false;//Should not happen
@@ -184,7 +197,7 @@ bool TreeItemRef::validChild(__attribute__((unused)) const QString &name,Q_ListV
 
 //See TreeItemAbstract for description of this virtual method
 bool TreeItemRef::deepReload(__attribute__((unused)) const QString &childName,Q_ListViewItem *oldItem) {
- CPdf* pdf=obj->getPdf();
+ boost::shared_ptr<CPdf> pdf=obj->getPdf().lock();
  if (!pdf) return false; //No document opened -> cannot parse references
  CRef* cref=dynamic_cast<CRef*>(obj.get());
  IndiRef ref;
