@@ -1,26 +1,33 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
+/*
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
-// vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
-/* 
- * =====================================================================================
- *        Filename:  carray.h
- *         Created:  01/18/2006 
- *          Author:  jmisutka (06/01/19), 
- * =====================================================================================
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
  */
+// vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
 #ifndef _CARRAY_H
 #define _CARRAY_H
 
 // all basic includes
-#include "static.h"
-#include "cobjectsimple.h"
+#include "kernel/static.h"
+#include "kernel/cobjectsimple.h"
 
 
 //=====================================================================================
@@ -58,7 +65,7 @@ class CArray : noncopyable, public IProperty
 public:
 	typedef std::vector<boost::shared_ptr<IProperty> > Value; 
 	typedef const std::string&		 				 WriteType; 
-	typedef unsigned int	 						 PropertyId;
+	typedef size_t			 						 PropertyId;
 	typedef observer::ComplexChangeContext<IProperty, PropertyId> CArrayComplexObserverContext;
 
 	/** 
@@ -83,7 +90,7 @@ public:
 	 * @param o		Xpdf object. 
 	 * @param rf	Indirect id and gen id.
 	 */
-	CArray (CPdf& p, Object& o, const IndiRef& rf);
+	CArray (boost::weak_ptr<CPdf> p, Object& o, const IndiRef& rf);
 
 	/**
 	 * Constructor.
@@ -188,7 +195,7 @@ public:
 	 * Set pdf to this object and its children.
 	 * @param pdf New pdf.
 	 */
-	virtual void setPdf (CPdf* pdf);
+	virtual void setPdf (boost::weak_ptr<CPdf> pdf);
 
 	/**
 	 * Set ref to this object and its children.
@@ -247,7 +254,7 @@ public:
 	 * Destructor
 	 */
 	~CArray () {}
-	
+		
 
 	//
 	// Helper methods
@@ -378,7 +385,7 @@ template <typename T> void complexValueToString (const typename T::Value& val, s
  */
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
-getSimpleValueFromArray (const boost::shared_ptr<CArray>& array, size_t position)__attribute__((deprecated));
+getSimpleValueFromArray (const boost::shared_ptr<CArray>& array, size_t position) DEPRECATED;
 
 // gcc doesn't like __attribute__ in function definition
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
@@ -417,7 +424,7 @@ getSimpleValueFromArray (const boost::shared_ptr<CArray>& array, size_t position
 /** \copydoc getSimpleValueFromArray */
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
-getSimpleValueFromArray (const boost::shared_ptr<IProperty>& array, size_t position)__attribute__((deprecated));
+getSimpleValueFromArray (const boost::shared_ptr<IProperty>& array, size_t position) DEPRECATED;
 	
 // gcc doesn't like __attribute__ in function definition
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
@@ -508,7 +515,7 @@ getRefFromArray (const IP& ip, size_t position)
  */
 template<typename Value, typename ItemType, PropertyType ItemPType>
 inline void
-setSimpleValueInArray (const CArray& array, size_t position, const Value& val)__attribute__((deprecated));
+setSimpleValueInArray (const CArray& array, size_t position, const Value& val) DEPRECATED;
 
 // gcc doesn't like __attribute__ in function definition
 template<typename Value, typename ItemType, PropertyType ItemPType>
@@ -551,7 +558,7 @@ setSimpleValueInArray (const CArray& array, size_t position, const typename Item
 /** \copydoc setSimpleValueInArray */
 template<typename Value, typename ItemType, PropertyType ItemPType>
 inline void
-setSimpleValueInArray (const IProperty& array, size_t position, const Value& val)__attribute__((deprecated));
+setSimpleValueInArray (const IProperty& array, size_t position, const Value& val) DEPRECATED;
 	
 // gcc doesn't like __attribute__ in function definition
 template<typename Value, typename ItemType, PropertyType ItemPType>
@@ -627,7 +634,7 @@ setDoubleInArray (const IP& ip, size_t position, double val)
  */
 template<typename ItemType, PropertyType ItemPType>
 inline boost::shared_ptr<ItemType>
-getTypeFromArray (const boost::shared_ptr<CArray>& array, size_t pos)__attribute__((deprecated));
+getTypeFromArray (const boost::shared_ptr<CArray>& array, size_t pos) DEPRECATED;
 
 // gcc doesn't like __attribute__ in function definition
 template<typename ItemType, PropertyType ItemPType>
@@ -688,7 +695,7 @@ getTypeFromArray (const boost::shared_ptr<CArray>& array, size_t pos)
 /** \copydoc getTypeFromArray */
 template<typename ItemType, PropertyType ItemPType>
 inline boost::shared_ptr<ItemType>
-getTypeFromArray (const boost::shared_ptr<IProperty>& array, size_t pos)__attribute__((deprecated));
+getTypeFromArray (const boost::shared_ptr<IProperty>& array, size_t pos) DEPRECATED;
 
 // gcc doesn't like __attribute__ in function definition
 template<typename ItemType, PropertyType ItemPType>
