@@ -1,25 +1,32 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
- *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
-// vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
 /*
- * =====================================================================================
- *        Filename:  cdict.cc
- *         Created:  02/02/2006 
- *          Author:  jmisutka (06/01/19), 
- * 			
- * =====================================================================================
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
  */
+// vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
 
-#include "static.h"
-#include "cdict.h"
-#include "cpdf.h"
+#include "kernel/static.h"
+#include "kernel/cdict.h"
+#include "kernel/cpdf.h"
+#include "kernel/factories.h"
 
 //=====================================================================================
 namespace pdfobjects {
@@ -39,9 +46,8 @@ using namespace boost;
 //
 // Protected constructor
 //
-CDict::CDict (CPdf& p, Object& o, const IndiRef& rf) : IProperty (&p,rf) 
+CDict::CDict (boost::weak_ptr<CPdf> p, Object& o, const IndiRef& rf) : IProperty (p,rf) 
 {
-	
 	// Build the tree from xpdf object
 	utils::complexValueFromXpdfObj<pDict,Value&> (*this, o, value);
 }
@@ -51,7 +57,6 @@ CDict::CDict (CPdf& p, Object& o, const IndiRef& rf) : IProperty (&p,rf)
 //
 CDict::CDict (Object& o)
 {
-	
 	// Build the tree from xpdf object
 	utils::complexValueFromXpdfObj<pDict,Value&> (*this, o, value);
 }
@@ -100,13 +105,11 @@ CDict::getProperty (PropertyId id) const
 	DictIdxComparator cmp (id);
 	Value::const_iterator it = value.begin();
 	for (; it != value.end(); ++it)
-	{
-			if (cmp (*it))
-					break;
-	}
+		if (cmp (*it))
+			break;
 
 	if (it == value.end())
-			throw ElementNotFoundException ("", "");
+		throw ElementNotFoundException ("", "");
 	
 	shared_ptr<IProperty> ip = cmp.getIProperty ();
 
@@ -125,7 +128,7 @@ CDict::getProperty (PropertyId id) const
 //
 //
 void 
-CDict::setPdf (CPdf* pdf)
+CDict::setPdf (boost::weak_ptr<CPdf> pdf)
 {
 	// Set pdf to this object
 	IProperty::setPdf (pdf);
@@ -170,10 +173,8 @@ CDict::delProperty (PropertyId id)
 	DictIdxComparator cmp (id);
 	Value::iterator oldit = value.begin();
 	for (; oldit != value.end(); ++oldit)
-	{
-			if (cmp (*oldit))
-					break;
-	}
+		if (cmp (*oldit))
+			break;
 	
 	if (oldit == value.end())
 		throw ElementNotFoundException ("CDict", "item not found");
@@ -201,11 +202,13 @@ CDict::delProperty (PropertyId id)
 		}
 
 		// Be sure
-		oldip->setPdf (NULL);
+		oldip->setPdf (boost::shared_ptr<CPdf>());
 		oldip->setIndiRef (IndiRef());
 
 	}else
-		{ assert (!hasValidRef (this)); }
+	{ 
+		assert (!hasValidRef (this)); 
+	}
 }
 
 
@@ -220,6 +223,12 @@ CDict::addProperty (const string& propertyName, const IProperty& newIp)
 	// Check whether we can make the change
 	this->canChange();
 
+	// check whether we contain such a property
+	//	-- pdf spec allows objects having two identical names, we forbid it!!!
+	//		-- e.g. because of double contents entry etc..
+	if (containsProperty(propertyName))
+		throw CObjInvalidObject ();
+
 	// Clone the added property
 	shared_ptr<IProperty> newIpClone = newIp.clone ();
 	if (newIpClone)
@@ -282,10 +291,8 @@ CDict::setProperty (PropertyId id, IProperty& newIp)
 	DictIdxComparator cmp (id);
 	Value::iterator it = value.begin();
 	for (; it != value.end(); ++it)
-	{
 		if (cmp (*it))
-				break;
-	}
+			break;
 
 	// Check the bounds, if fails add it
 	if (it == value.end())
@@ -325,11 +332,13 @@ CDict::setProperty (PropertyId id, IProperty& newIp)
 		}
 
 		// Be sure
-		oldIp->setPdf (NULL);
+		oldIp->setPdf (boost::shared_ptr<CPdf>());
 		oldIp->setIndiRef (IndiRef());
 	
 	}else
-		{ assert (!hasValidRef (this)); }
+	{ 
+		assert (!hasValidRef (this)); 
+	}
 
 	// Set mode only if pdf is valid
 	_setMode (newIpClone,id);
@@ -349,18 +358,30 @@ CDict::setProperty (PropertyId id, IProperty& newIp)
 CDict::_makeXpdfObject () const
 {
 	//kernelPrintDbg (debug::DBG_DBG,"_makeXpdfObject");
-	
-	string rpr;
-	getStringRepresentation (rpr);
+	::Object * dictObj = XPdfObjectFactory::getInstance();
 
-	::Object* o = NULL;
-	if (hasValidPdf (this))
-		o = utils::xpdfObjFromString (rpr, this->getPdf()->getCXref());
-	else
-		o = utils::xpdfObjFromString (rpr);
-	assert (o->isDict());
+	// We need to be carefull with pdf from property. Some
+	// properties (usually artificial ones) can be without
+	// any bound PDF instance. Nevertheless NULL xref is ok for
+	// Object hierarchy (see Object::fetch method)
+	XRef *xref = NULL;
+	boost::shared_ptr<CPdf> pdf = getPdf().lock();
+	if(pdf)
+		xref = pdf->getCXref();
+	dictObj->initDict(xref);
+	::Dict * dict = dictObj->getDict();
 
-	return o;
+	Value::const_iterator it = value.begin();
+	for (; it != value.end(); ++it)
+	{
+		shared_ptr<IProperty> prop = it->second;
+		Object * propObj = prop->_makeXpdfObject();
+		dict->add(copyString((it->first).c_str()), propObj);
+		gfree(propObj);
+	}
+	assert(static_cast<unsigned int>(dict->getLength()) == getPropertyCount());
+
+	return dictObj;
 }
 
 
@@ -398,7 +419,8 @@ void
 CDict::_setMode (shared_ptr<IProperty> ip, PropertyId id) const
 {
 	configuration::ModeController* modecontroller = NULL;
-	if (hasValidPdf (this) && (NULL != (modecontroller=this->getPdf()->getModeController())))
+	shared_ptr<CPdf> p = this->getPdf().lock();
+	if ( p && (NULL != (modecontroller=p->getModeController())))
 	{
 		assert (modecontroller);		
 		PropertyMode mode;
@@ -411,8 +433,8 @@ CDict::_setMode (shared_ptr<IProperty> ip, PropertyId id) const
 		Value::const_iterator it = value.begin();
 		for (; it != value.end(); ++it)
 		{
-				if (cmp (*it))
-						break;
+			if (cmp (*it))
+				break;
 		}
 		if (it == value.end())
 		{ // No type found
