@@ -1,30 +1,35 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
+/*
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
-// vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
-/* 
- * =====================================================================================
- *        Filename:  cdict.h
- *     Description:  Header file containing definition of IProperty and CObject classes.
- *         Created:  01/18/2006 
- *          Author:  jmisutka (06/01/19), 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
  *
- * =====================================================================================
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
  */
+// vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
 #ifndef _CDICT_H
 #define _CDICT_H
 
 // all basic includes
-#include "static.h"
-#include "iproperty.h"
-#include "cobjectsimple.h"
-#include "carray.h"
+#include "kernel/static.h"
+#include "kernel/iproperty.h"
+#include "kernel/cobjectsimple.h"
+#include "kernel/carray.h"
 
 
 //=====================================================================================
@@ -65,8 +70,8 @@ class CDict : noncopyable, public IProperty
 	
 public:
 	typedef std::list<std::pair<std::string, boost::shared_ptr<IProperty> > > Value; 
-	typedef const std::string& 			WriteType; 
-	typedef const std::string& 			PropertyId;
+	typedef const std::string& WriteType; 
+	typedef const std::string& PropertyId;
 	typedef observer::ComplexChangeContext<IProperty, PropertyId> CDictComplexObserverContext;
 
 	/** 
@@ -91,7 +96,7 @@ public:
 	 * @param o		Xpdf object. 
 	 * @param rf	Indirect id and gen id.
 	 */
-	CDict (CPdf& p, Object& o, const IndiRef& rf);
+	CDict (boost::weak_ptr<CPdf> p, Object& o, const IndiRef& rf);
 
 	/**
 	 * Constructor.
@@ -106,7 +111,7 @@ public:
 	/** 
 	 * Public constructor. This object will not be associated with a pdf.
 	 */
-	CDict () {};
+	CDict () {}
 
 
 	//
@@ -131,7 +136,9 @@ protected:
 	 * @return New complex object.
 	 */
 	virtual CDict* _newInstance () const
-		{ return new CDict; }
+	{ 
+		return new CDict; 
+	}
 	
 	//
 	// Get methods
@@ -143,7 +150,7 @@ public:
 	 *
 	 * @return Type of this property.
 	 */
-	virtual PropertyType getType () const {return type;};
+	virtual PropertyType getType () const {return type;}
 	
 
 	/**
@@ -165,7 +172,7 @@ public:
 	{
 		//kernelPrintDbg (debug::DBG_DBG, "getPropertyCount(" << debug::getStringType<Tp>() << ") = " << value.size());
 		return value.size();
-	};
+	}
  
 
 	/**
@@ -207,7 +214,7 @@ public:
 	 *
 	 */
 	PropertyType getPropertyType (PropertyId id) const 
-		{ return getProperty(id)->getType(); };
+		{ return getProperty(id)->getType(); }
 
 	
 	//
@@ -218,7 +225,7 @@ public:
 	 * Set pdf to this object and its children.
 	 * @param pdf New pdf.
 	 */
-	virtual void setPdf (CPdf* pdf);
+	virtual void setPdf (boost::weak_ptr<CPdf> pdf);
 
 	/**
 	 * Set ref to this object and its children.
@@ -246,20 +253,17 @@ public:
 	 * the cloned object is added.
 	 * Indicate that this object has changed and return the pointer to the cloned object.
 	 *
-	 * @param newIp 		New property.
+	 * If a property with the same name is already in this dictionary an exception is thrown.
+	 *
+	 * @param propertyName property name
+	 * @param newIp new property
 	 * @return Pointer to the new property.
 	 *
 	 * \exception OutOfRange Thrown when property not found.
-	 */
-	boost::shared_ptr<IProperty> addProperty (const IProperty& newIp);
-	
-	/**
-	 * \copydoc addProperty
-	 * @param propertyName 	Name of the created property.
+	 * \exception CObjInvalid When the object is invalid
 	 */
 	boost::shared_ptr<IProperty> addProperty (const std::string& propertyName, const IProperty& newIp);
 
-	
 	/**
 	 * Remove property from dictionary. 
 	 *
@@ -277,7 +281,7 @@ public:
 	/**
 	 * Destructor
 	 */
-	~CDict () {};
+	~CDict () {}
 	
 
 	//
@@ -333,7 +337,7 @@ private:
 	 * @param context Context in which a change occured.
 	 */
 	void _objectChanged (boost::shared_ptr<IProperty> newValue, 
-						 boost::shared_ptr<const ObserverContext> context);
+			boost::shared_ptr<const ObserverContext> context);
 
 	//
 	// Mode interface
@@ -357,7 +361,7 @@ public:
 	void _getAllChildObjects (Storage& store) const
 	{
 		Value::const_iterator it = value.begin ();
-		for	(; it != value.end (); ++it)
+		for(; it != value.end (); ++it)
 			store.push_back ((*it).second);
 	}
 
@@ -389,9 +393,9 @@ private:
 	boost::shared_ptr<IProperty> ip;
 
 public:
-	DictIdxComparator (const std::string& s) : str(s) {};
+	DictIdxComparator (const std::string& s) : str(s) {}
 		
-	boost::shared_ptr<IProperty> getIProperty () {return ip;};
+	boost::shared_ptr<IProperty> getIProperty () {return ip;}
 		
 	bool operator() (const CDict::Value::value_type& item)
 	{	
@@ -402,7 +406,7 @@ public:
 		}
 			
 		return false;
-	};
+	}
 };
 
 
@@ -451,25 +455,82 @@ void dictFromXpdfObj (CDict& resultDict, ::Object& dict);
  */
 size_t stringToCharBuffer(Object & stringObject, CharBuffer & outputBuf);
 
+/** Helper function which returns data from given stream.
+ * @param str Stream to read (until it returns EOF).
+ * @param dictLength Initial size reported by the stream dictionary.
+ * @param size Output size of the buffer.
+ *
+ * This is low-level function which simply reads given stream from its 
+ * beginning until EOF. It can be used to retrieve data from whatever
+ * Stream filter stack layer. E.g. if we want to get encoded data we
+ * can simply give stream->getBaseStream() as parameter and this function
+ * will return buffer as it is written in the file.
+ *
+ * @return Buffer with stream data (size will contain number of bytes
+ * stored) or NULL on error.
+ */
+unsigned char* bufferFromStream(Stream& str, size_t dictLength, size_t& size);
+
+/** Helper function for removing filters from the given stream object.
+ * @param obj Stream object.
+ * @param size Size of returned buffer.
+ * @return Buffer with raw stream data or NULL on error.
+ *
+ * If the given obj used some filters, they are all decoded to get orignal
+ * data and the stream dictionary is updated so that it doesn't contain
+ * any filters. 
+ * <br>
+ * Stream object stored in the given obj is not touched (because it doesn't
+ * give much sense - it returns decoded data anyway, it doesn't use dictionary
+ * to find out how to decode). Nevertheless this operation may be considered 
+ * harmfull for later usage of given the object in the xpdf code paths!
+ */
+unsigned char* convertStreamToDecodedData (Object& obj, size_t& size);
+
+/** Function to be used for data extracting from the given stream object.
+ * Note that implementation can apply additional filters to the stream
+ * data currently stored in the obj.stream and update object accordingly
+ * (change Filter and associated entry). Length entry doesn't have to 
+ * be changed by the implementation as it is supposed to be updated by 
+ * caller (because we want to prevent useless object updating when a
+ * chain of this function is called for multiple filters).
+ * <br>
+ * Function is called from the streamToCharBuffer function to have transparent
+ * access to the stream data without any knowledge about filters.
+ * bufferFromStreamData is the referencial implementation which provides
+ * encoded data without any filters applied (all other implementations
+ * can use this function as the base and convert returned buffer into
+ * their represenation).
+ *
+ * @param Stream object.
+ * @param size Number of bytes written to the returned buffer.
+ * @return Buffer (size bytes) with data (must be deallocated by caller) or 
+ * NULL on error.
+ */
+typedef unsigned char* (*stream_data_extractor)(Object& obj, size_t& size);
+
 /** Makes a valid pdf indirect object representation of stream object.
  * @param streamObject Xpdf object representing stream.
  * @param ref Reference for this indirect object.
  * @param outputBuf Output byte buffer containing complete representation.
- * @param asIndirect Flag for indirect output.
+ * @param extractor Function to be used to extract data from the object's 
+ * 	stream.
  *
- * Allocates and fill buffer in given outputBuf with pdf object format
+ * Allocates and fills buffer in given outputBuf with pdf object format
  * representation of given stream object. Moreover adds indirect header and
- * footer if asIndirect parameter is true. 
+ * footer if ref is non NULL. 
+ * <br>
+ * If you want some filters to be used to encode stream data, use extractor
+ * function parameter for this purpose. bufferFromStreamData used by default
+ * returns stream data without any filters applied.
  * <br>
  * Given buffer may contain NUL bytes inside. Caller should consume number of
  * returned bytes from outputBuf.
- * <br>
- * Reference parameter is ignored if asIndirect is false, because it is used
- * only for indirect object header.
  * 
  * @return number of bytes used in outputBuf or 0 if problem occures.
  */
-size_t streamToCharBuffer (Object & streamObject, Ref ref, CharBuffer & outputBuf, bool asIndirect=true);
+size_t streamToCharBuffer (Object & streamObject, Ref* ref, CharBuffer & outputBuf, 
+		stream_data_extractor extractor);
 	
 /**
  * Convert xpdf object to string
@@ -479,15 +540,13 @@ size_t streamToCharBuffer (Object & streamObject, Ref ref, CharBuffer & outputBu
  */
 void xpdfObjToString (Object& obj, std::string& str);
 
-
 /**
  * Get string from xpdf stream object.
  *
  * @param str Output string.
  * @param obj Xpdf stream object.
  */
-void
-getStringFromXpdfStream (std::string& str, ::Object& obj);
+void getStringFromXpdfStream (std::string& str, ::Object& obj);
 
 
 //=========================================================
@@ -517,7 +576,7 @@ getStringFromXpdfStream (std::string& str, ::Object& obj);
  */
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
-getSimpleValueFromDict (const boost::shared_ptr<CDict>& dict, const std::string& id)__attribute__((deprecated));
+getSimpleValueFromDict (const boost::shared_ptr<CDict>& dict, const std::string& id) DONOTUSE;
 
 /** \copydoc getSimpleValueFromDict */
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
@@ -556,7 +615,7 @@ getSimpleValueFromDict (const boost::shared_ptr<CDict>& dict, CDict::PropertyId
 /** \copydoc getSimpleValueFromDict */
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
-getSimpleValueFromDict (const CDict& dict, const std::string& id)__attribute__((deprecated));
+getSimpleValueFromDict (const CDict& dict, const std::string& id) DONOTUSE;
 
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
@@ -587,7 +646,7 @@ getSimpleValueFromDict (const CDict& dict, const std::string& id)
 /** \copydoc getSimpleValueFromDict */
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
-getSimpleValueFromDict (const boost::shared_ptr<IProperty>& dict, const std::string& id)__attribute__((deprecated));
+getSimpleValueFromDict (const boost::shared_ptr<IProperty>& dict, const std::string& id) DONOTUSE;
 	
 template<typename SimpleValueType, typename ItemType, PropertyType ItemPType>
 inline SimpleValueType
@@ -647,13 +706,17 @@ getDoubleFromDict (const IP& ip, const std::string& id)
 template<typename IP>
 inline std::string
 getStringFromDict (const IP& ip, const std::string& id)
-	{ return getSimpleValueFromDict<CString> (ip, id);}
+{ 
+	return getSimpleValueFromDict<CString> (ip, id);
+}
 
 /** Get name from dictionary. */
 template<typename IP>
 inline std::string
 getNameFromDict (const IP& ip, const std::string& id)
-	{ return getSimpleValueFromDict<CName> (ip, id);}
+{ 
+	return getSimpleValueFromDict<CName> (ip, id);
+}
 
 //=========================================================
 //	CDict "set value" helper methods
@@ -670,7 +733,7 @@ getNameFromDict (const IP& ip, const std::string& id)
  */
 template<typename Value, typename ItemType, PropertyType ItemPType>
 inline void
-setSimpleValueInDict (const CDict& dict, const std::string& name, const Value& val)__attribute__((deprecated));
+setSimpleValueInDict (const CDict& dict, const std::string& name, const Value& val) DONOTUSE;
 
 template<typename Value, typename ItemType, PropertyType ItemPType>
 inline void
@@ -715,7 +778,7 @@ setSimpleValueInDict (const CDict& dict, CDict::PropertyId name, const typename
 /** \copydoc setSimpleValueInDict */
 template<typename Value, typename ItemType, PropertyType ItemPType>
 inline void
-setSimpleValueInDict (const IProperty& dict, const std::string& name, const Value& val)__attribute__((deprecated));
+setSimpleValueInDict (const IProperty& dict, const std::string& name, const Value& val) DONOTUSE;
 	
 template<typename Value, typename ItemType, PropertyType ItemPType>
 inline void
@@ -751,7 +814,9 @@ setSimpleValueInDict (const IProperty& ip, CDict::PropertyId name, const typenam
 template<typename IP>
 inline void
 setIntInDict (const IP& ip, const std::string& name, int val)
-	{ setSimpleValueInDict<int, CInt, pInt> (ip, name, val);}
+{ 
+	setSimpleValueInDict<int, CInt, pInt> (ip, name, val);
+}
 
 /** Set	double in dictioary. */
 template<typename IP>
@@ -780,7 +845,7 @@ setDoubleInDict (const IP& ip, const std::string& name, double val)
  */
 template<typename ItemType, PropertyType ItemPType>
 inline boost::shared_ptr<ItemType>
-getTypeFromDictionary (const boost::shared_ptr<CDict>& dict, const std::string& key)__attribute__((deprecated));
+getTypeFromDictionary (const boost::shared_ptr<CDict>& dict, const std::string& key) DONOTUSE;
 
 template<typename ItemType, PropertyType ItemPType>
 inline boost::shared_ptr<ItemType>
@@ -834,7 +899,9 @@ getTypeFromDictionary (const boost::shared_ptr<CDict>& dict, CDict::PropertyId k
 	// Check the type
 	if (ItemType::type != ip->getType ())
 	{
-		utilsPrintDbg (debug::DBG_DBG, "wanted type " << ItemType::type << " got " << ip->getType () << " key[" << key << "]");
+		utilsPrintDbg (debug::DBG_DBG, "wanted type " << ItemType::type 
+				<< " got " << ip->getType () 
+				<< " key[" << key << "]");
 		std::string err= "getTypeFromDictionary() [" + key + "]";
 		throw ElementBadTypeException (err);
 	}
@@ -846,7 +913,7 @@ getTypeFromDictionary (const boost::shared_ptr<CDict>& dict, CDict::PropertyId k
 /** \copydoc getTypeFromDictionary */
 template<typename ItemType, PropertyType ItemPType>
 inline boost::shared_ptr<ItemType>
-getTypeFromDictionary (const boost::shared_ptr<IProperty>& dict, const std::string& key)__attribute__((deprecated));
+getTypeFromDictionary (const boost::shared_ptr<IProperty>& dict, const std::string& key) DONOTUSE;
 
 template<typename ItemType, PropertyType ItemPType>
 inline boost::shared_ptr<ItemType>
@@ -896,7 +963,9 @@ getTypeFromDictionary (const boost::shared_ptr<IProperty>& ip, CDict::PropertyId
 template<typename IP>
 inline boost::shared_ptr<CDict>
 getCDictFromDict (IP& ip, const std::string& key)
-	{return getTypeFromDictionary<CDict> (ip, key);}
+{
+	return getTypeFromDictionary<CDict> (ip, key);
+}
 
 /** 
  * Get array from dictionary. If it is an indirect object, fetch the object.
@@ -904,7 +973,9 @@ getCDictFromDict (IP& ip, const std::string& key)
 template<typename IP>
 inline boost::shared_ptr<CArray>
 getCArrayFromDict (IP& ip, const std::string& key)
-	{return getTypeFromDictionary<CArray> (ip, key);}
+{
+	return getTypeFromDictionary<CArray> (ip, key);
+}
 
 
 /** 
@@ -912,7 +983,9 @@ getCArrayFromDict (IP& ip, const std::string& key)
  */
 inline boost::shared_ptr<CDict>
 getCDictFromDict (boost::shared_ptr<CDict> dict, const std::string& key)
-	{return getTypeFromDictionary<CDict> (dict, key);}
+{
+	return getTypeFromDictionary<CDict> (dict, key);
+}
 
 
 //=====================================================================================
