@@ -1,93 +1,33 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
+/*
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
+ */
 // vim:tabstop=4:shiftwidth=4:noexpandtab:textwidth=80
 
-/*
- * $RCSfile: pdfwriter.h,v $
- *
- * $Log: pdfwriter.h,v $
- * Revision 1.12  2007/04/25 18:24:35  bilboq
- *
- * applied patch from Martin Michlmayr to fix compilation with gcc-4.3
- *
- * fixed some typos in comments
- *
- * Revision 1.11  2007/02/04 20:17:02  mstsxfx
- * Common Licence comment for all cc and h files available in doc/licence_header
- * file and its content to all cc and h files in src/{gui,kernel,utils}
- * directories.
- * Xpdf code, QSA and QOutputDevice code are not touched
- *
- * Revision 1.10  2006/06/29 20:00:08  hockm0bm
- * doc updated
- *
- * Revision 1.9  2006/06/18 12:04:42  hockm0bm
- * obsevers code clean up and consolidation
- *
- * Revision 1.8  2006/06/17 18:34:52  hockm0bm
- * Refactoring changes
- *
- * * IObserverHandler
- *         -renamed to ObserverHandler - it is not interface
- *         - BasicObserverContext, CDictComplexObserverContext,
- *           CArrayComplexObserverContext removed - it doesn't have to know
- *           anything about existing context types
- * * CObjectSimple, CDict, CArray, CStream, CPage, CAnnotation, CContentStream
- *         - each provides typedef with produced change context
- *
- * Revision 1.7  2006/06/05 22:28:29  hockm0bm
- * * IProgressBar interface added
- * * ProgressObserver implemented
- *
- * Revision 1.6  2006/05/24 19:30:31  hockm0bm
- * OldStylePdfWriter::maxObjNum is initialized in constructor (it wasn't)
- *
- * Revision 1.5  2006/05/23 19:04:03  hockm0bm
- * OldStylePdfWriter::writeTrailer
- *         - signature changed returns position after stored xref section
- *         - trims averything behind stored data
- *
- * Revision 1.4  2006/05/17 19:42:00  hockm0bm
- * EOFMARKER is extern now
- *
- * Revision 1.3  2006/05/16 17:57:28  hockm0bm
- * * infrastructure for obserevers usable for IPdfWriter
- *         - OperationStep, OperationScope structures
- *         - PdfWriterObserver and PdfWriterObserverContext types
- * * OldStylePdfWriter uses observers to provide progress information
- * * writeTrailer method changed
- *         - lastXref parameter replaced by PrevSecInfo structure
- *         - Trailer Prev field removed if PrevSecInfo::xrefPos==0
- *         - Trailer Size field is set to proper value
- *
- * Revision 1.2  2006/05/09 20:10:55  hockm0bm
- * * doc update
- * * writeContent some checking added
- *         - duplicated entries are ignored
- *         - NULL entries are ignored
- *
- * Revision 1.1  2006/05/08 20:12:19  hockm0bm
- * * abstract IPdfWriter class for pdf content writers
- * * OldStylePdfWriter implementation of IPdfWriter
- * * pdf key words defined here now
- *
- *
- */
 #ifndef _PDF_WRITER_H_
 #define _PDF_WRITER_H_
 
-#include<vector>
-#include<map>
-#include "streamwriter.h"
-#include "utils/objectstorage.h"
-#include "utils/observer.h"
+#include "kernel/static.h"
+#include "kernel/cxref.h"
 
 /** Header of pdf file.
  * This string should be appended by pdf version number.
@@ -116,6 +56,8 @@ extern const char * STARTXREF_KEYWORD;
  */
 extern const char * EOFMARKER;
 
+class StreamWriter;
+
 namespace pdfobjects {
 
 namespace utils {
@@ -270,17 +212,169 @@ public:
 	}
 };
 
+
+/** Base class for filter stream writers.
+ * Note that this class - unlike StreamWriter classes defined in streamwriter.h 
+ * file is not based on xpdf Stream object. Its purpose is to help IPdfWriter
+ * class to write stream object into the pdf document.
+ * <br>
+ * Use factory getInstance method to get proper filter stream writer 
+ * implementation for given stream object. Default writer is used if no such 
+ * implementation is available.
+ */
+class FilterStreamWriter
+{
+public:
+	typedef std::vector<boost::shared_ptr<FilterStreamWriter> > WritersList;
+protected:
+	/** List of the registered filter stream writers.
+	 */
+	static WritersList writers;
+
+	/** Default filter stream writer used if no registered can be used.
+	 */
+	static boost::shared_ptr<FilterStreamWriter> defaultWriter;
+public:
+	/** Finds proper filter stream writer for given obj.
+	 * If no registered appropriate writer exists, defaultWriter is tried. 
+	 * If neither defaultWriter is ok, NullFilterStreamWriter is used which 
+	 * writes stream object as it is without no filters.
+	 *
+	 * @param objStream Stream object.
+	 * @return Appropriate filter stream writer (never NULL).
+	 */
+	static boost::shared_ptr<FilterStreamWriter> getInstance(Object& objStream);
+
+	/** Adds new filter writer to the registered.
+	 * @param streamwriter Filter stream writer implementation.
+	 */
+	static void registerFilterStreamWriter(boost::shared_ptr<FilterStreamWriter> streamWriter);
+
+	/** Removes already registerd filter writer.
+	 * @param streamWriter Filter stream writer to remove.
+	 */
+	static void unregisterFilterStreamWriter(boost::shared_ptr<FilterStreamWriter> streamWriter);
+
+	/** Removes all registered filter writers.
+	 */
+	static void clearAllFilterStreamWriters();
+
+	/** Sets default stream writer.
+	 * @param streamWriter Writer to be set.
+	 */
+	static void setDefaultStreamWriter(boost::shared_ptr<FilterStreamWriter> defaultWriter);
+
+	/** Checks whether this filter writer is able to handle given object.
+	 * Implementation is absolutely free on which criteria it decides whether
+	 * it is or not able to handle this object. Usual way, however, is to check
+	 * Filter entry and check if curren writer is able to write data in given
+	 * format. @see compress method for more information
+	 * @param obj Object to be written.
+	 * @return true if able, false otherwise.
+	 */
+	virtual bool supportObject(Object& obj)const =0;
+
+	/** Writes given stream object to the output stream.
+	 * Implementation has to follow pdf specification in format of the data
+	 * writen in the stream. Nevertheless it is absolutely free in how it does it.
+	 * It can modify given object to use those filters (and all associated 
+	 * parameters) which are then used when data are written.
+	 * @param obj Object to write (must be stream).
+	 * @param ref Indirect reference for object (NULL for direct object).
+	 * @param outStream Output stream where to put data.
+	 */
+	virtual void compress(Object& obj, Ref* ref, StreamWriter& outStream)const =0;
+};
+
+/** Stream writer implementation with no filters.
+ * This writer will write stream data as is with all original
+ * filters and no modifications to original data.
+ */
+class NullFilterStreamWriter: public FilterStreamWriter
+{
+	static boost::shared_ptr<NullFilterStreamWriter> instance;
+public:
+	static boost::shared_ptr<NullFilterStreamWriter> getInstance();
+
+	/** Checks whether given object is supported.
+	 * @return allways true as it can write all stream objects.
+	 */
+	virtual bool supportObject(UNUSED_PARAM Object& obj)const;
+
+	/** Extracts stream data without any decoding.
+	 */
+	static unsigned char * null_extractor(Object&obj, size_t& size);
+
+	/** Writes given stream object to the stream.
+	 * @param obj Stream object.
+	 * @param ref Indirect reference for object (NULL if direct).
+	 * @param outStream Stream where to write data.
+	 *
+	 * Uses streamToCharBuffer with null_extractor extractor.
+	 */
+	virtual void compress(Object& obj, Ref* ref, StreamWriter& outStream)const;
+};
+
+/** Implementation of FlateDecode filter stream writer.
+ * It is based on zlib implementation of default deflate method.
+ */
+class ZlibFilterStreamWriter: public FilterStreamWriter
+{
+	/** Shared writer instance */
+	static boost::shared_ptr<ZlibFilterStreamWriter> instance;
+
+	/** Updates given stream object with the applied fiter data.
+	 * @param obj Stream object.
+	 *
+	 * Only for internal use of ZlibFilterStreamWriter class.
+	 */
+	static void update_dict(Object& obj);
+public:
+	static boost::shared_ptr<ZlibFilterStreamWriter> getInstance();
+
+	/** Checks whether given stream object is supported by this writer.
+	 * @param obj Stream object.
+	 * @return true if no filter FlateDecode are used.
+	 */
+	virtual bool supportObject(Object& obj)const;
+
+	/** Compress given buffer with deflate method.
+	 * @param in Input buffer.
+	 * @param in_size Input buffer size.
+	 * @param size Size of the output buffer data.
+	 * @return allocated buffer with the size data bytes or NULL on failure.
+	 *
+	 * Uses zlib interface to deflate given data.
+	 */
+	static unsigned char* deflate_buffer(unsigned char * in, size_t in_size, size_t& size);
+
+	/** Stream data extractor implementation for streamToCharBuffer function.
+	 * @param obj Stream object.
+	 * @param size Size of the returned buffer data.
+	 * @return allocated buffer with data or NULL on failure.
+	 * 
+	 * Uses bufferFromStreamData to get raw data without any filters,
+	 * compresses returned buffer with the deflate_buffer function and 
+	 * updates given stream object's dictionary to contain proper filter 
+	 * data.
+	 */
+	static unsigned char* deflate(Object& obj, size_t& size);
+
+	virtual void compress(Object& obj, Ref* ref, StreamWriter& outStream)const;
+};
+
 /** Interface for pdf content writer.
  *
  * Implementator knows how to put data to the file to create correct pdf
  * content.
  * <br>
- * Pdf content is written in 2 phases:
+ * Pdf content is written in 2 or 3 phases:
  * <ul>
- * <li>objects writing - when writeContent method is called (one or more times).
- * This data forms revision.
- * <li>cross reference & trailer writing - when all collected data are used for
- * cross reference and trailer to finish PDF revision.
+ * <li>header writing - only if you are writing complete document (writeHeader).
+ * <li>objects writing - writes given set of objects (can be called several times, 
+ * each time with the batch of objects). All this data form single revision.
+ * <li>cross reference & trailer writing - writes cross reference data and trailer 
+ * which ends current revision. 
  * </ul>
  *
  * So each sequence of writeContent [, writeContent]* , writeTrailer forms new 
@@ -334,14 +428,29 @@ public:
 		 */
 		size_t xrefPos;
 
-		/** Number of objects in previous section.
-		 *
-		 * If no previous section is available value should be 0.
+		/** Number of entries in previous section.
 		 */
-		size_t objNum;
+		size_t entriesNum;
 	};
 
-	virtual ~IPdfWriter(){}
+	virtual ~IPdfWriter()
+	{
+#ifdef OBSERVER_DEBUG
+		std::cerr << this <<":pdfWriter:";
+		if(observers.size())
+			dumpObservers();
+		std::cerr << std::endl;
+#endif
+	}
+
+	/** Writes PDF header to the given stream.
+	 * @param version Version of the PDF standard used for this document.
+	 * @param stream Stream writer where to write.
+	 *
+	 * Moves the current position in the stream at the begginning before
+	 * writing.
+	 */
+	virtual void writeHeader(const char* version, StreamWriter &stream);
 
 	/** Puts all objects to given stream.
 	 * @param objectList List of objects to store.
@@ -392,7 +501,7 @@ class OldStylePdfWriter: public IPdfWriter
 	/** Type for offset table.
 	 * Mapping from reference to file offset of indirect object.
 	 */
-	typedef std::map< ::Ref, size_t, RefComparator> OffsetTab;
+	typedef std::map< ::Ref, size_t, xpdf::RefComparator> OffsetTab;
 
 	/** Offset table.
 	 *
@@ -428,19 +537,19 @@ public:
 	 * This value is used in ScopedChangeContext's task field in writeContent
 	 * method. User of the instance can change this value directly.
 	 */
-	std::string CONTENT;
+	static const std::string CONTENT;
 
 	/** String for context task in writeTrailer.
 	 * This value is used in ScopedChangeContext's task field in writeTrailer
 	 * method. User of the instance can change this value directly.
 	 */
-	std::string TRAILER;
+	static const std::string TRAILER;
 		
 	/** Initialize constructor.
 	 *
 	 * Initializes CONTENT and TRAILER fields to default values.
 	 */
-	OldStylePdfWriter():maxObjNum(0), CONTENT("Content phase"), TRAILER("XREF/TRAILER phase"){}
+	OldStylePdfWriter():maxObjNum(0){}
 
 	/** Writes given objects.
 	 * @param objectList List of objects to write.
@@ -522,6 +631,193 @@ public:
 	virtual void reset();
 };
 
+/** Helper data structure which keeps all file stream related data.
+ */
+struct FileStreamData 
+{
+	/** Stream writer.
+	 * Provides interface to write data to the stream.
+	 */
+	StreamWriter *stream;
+
+	/** File handle.
+	 *
+	 * We have to keep file handle separately from the stream
+	 * because the stream used for XRef doesn't allow to access its
+	 * file handle from outside and also doesn't provide proper
+	 * close functionality.
+	 */
+	FILE *file;
+};
+
+/** Helper class to deallocate a type which holds FileStreamData and
+ * needs to destroy it after its holder is destroyed.
+ *
+ * This object should be used as a destroyer for smart_ptr shared 
+ * pointer to accomplish after-destroy cleanup.
+ * <br>
+ * Currently only the file from FileStreamData structure is
+ * considered (closed by functor) because stream is handled by others.
+ * <br>
+ * Note that you have to make this class as a friend if the destructor
+ * is not available.
+ */
+template<typename T> class FileStreamDataDeleter
+{
+	FILE * file;
+public:
+	FileStreamDataDeleter(FileStreamData &data):file(data.file) {};
+
+	/** Deallocates given instance and cleans-up necessary 
+	 * FileStreamData fields.
+	 * @param instance Instance to be destroyed.
+	 */
+	void operator()(T *instance)
+	{
+		assert(instance);
+		delete instance;
+
+		if(fclose(file))
+		{
+			int err = errno;
+			kernelPrintDbg(debug::DBG_ERR, "Unable to close file handle (cause=\""
+					<<strerror(err) << "\"");
+		}
+	}
+};
+
+/** Generic class to PDF document writing.
+ * The main responsibility of this class is to provide unified interface
+ * for arbitrary PDF document writing (note that this limits to complete
+ * documents - so it doesn't include incremental update document writing).
+ * <br>
+ * Class is meant to be base type for other specialized document writers
+ * which are supposed to implement fillObjectList abstract class. All 
+ * other document structure related issues are already implemented in this
+ * class.
+ * <br>
+ * Note that the class doesn't write content of the file direct but it
+ * uses IPdfWriter provided in construtor or by setPdfWriter instead.
+ */
+class PdfDocumentWriter: public pdfobjects::CXref
+{
+	/** Number of objects that should be written in one batch.
+	 * This constant is used as the maxObjectCount parameter to 
+	 * fillObjectList method.
+	 */
+	static const int writeBatchCount = 1000;
+
+protected:
+	/** Pdf content writer implementator.
+	 *
+	 * All writing of pdf content is delegated to this object.
+	 */
+	IPdfWriter * pdfWriter;
+
+	/** Abstract method to provide objects to be written.
+	 * @param objectList List of objects to be filled.
+	 * @param maxObjectCount Maximum number of objects to be filled.
+	 *
+	 * This method is called by writeDocument until it returns no objects.
+	 * Implementation may clear the list before it adds new elements but
+	 * is must provide up to given maxObjectCount. Use maxObjectCount=0 
+	 * for unlimited number of objects.
+	 *
+	 * @return Number of objects added to given objectList.
+	 * @throw MalformedFormatExeption if the document content is not valid.
+	 */
+	virtual int fillObjectList(IPdfWriter::ObjectList &objectList, int maxObjectCount)=0;
+	
+	/** Opens output file and writes a new document to it.
+	 * @param fileName File to be opened.
+	 * 
+	 * Opens given file (in trucate mode) and delegates the rest to 
+	 * delinearize(FILE *) method. If given file doesn't exist, it will be
+	 * created. Finally closes file.
+	 * @return 0 on success, errno otherwise.
+	 * @throw NotImplementedException if document is encrypted.
+	 */
+	virtual int writeDocument(const char *fileName);
+
+	/** Writes a new document to the given file.
+	 * @param file Opened file handle where to write.
+	 *
+	 * Sets position to the file beginning and writes the same pdf header as in
+	 * the original stream. Then writes all objects provided by fillObjectList 
+	 * (calls this method repeatedly unless 0 objects are returned).
+	 * Finally stores xref and trailer section.
+	 * <br>
+	 * Caller is responsible for file handle closing.
+	 * <br>
+	 * NOTE that method doesn't check whether given file is same as the 
+	 * one used for input data. If it refers to same file, result is 
+	 * unpredictable.
+	 * <br>
+	 * Returns with erro (EINVAL) if no pdfWriter is specified (it is NULL).
+	 *
+	 * @return 0 if everything ok, otherwise value of error of the error.
+	 * @throw NotImplementedException if document is encrypted.
+	 * @throw MalformedFormatExeption if the input file is currupted.
+	 * 
+	 * @return 0 on success, errno otherwise.
+	 */
+	virtual int writeDocument(FILE *file);
+
+	/** Initialization constructor.
+	 * @param streamData Input stream data.
+	 * @param writer Pdf content writer.
+	 *
+	 * Uses CXref(BaseStream *) constructor and initializes file handle and 
+	 * pdfWriter with given one (pdfWriter has to be allocated by new operator,
+	 * because it is deallocated by delete in destructor - if NULL is provided,
+	 * writeDocument methods do nothing).
+	 *
+	 * @throw MalformedFormatExeption if file content is not valid pdf document.
+	 */
+	PdfDocumentWriter(FileStreamData &data, IPdfWriter *pdfWriter);
+
+public:
+	/** Opens file and creates StreamData.
+	 * @param fileName Name of the file to open.
+	 * 
+	 * This method should be preferred way to provide constructor
+	 * data parameter.
+	 * @return StreamData which contains opened stream and file handle or
+	 * NULL if not able to open file for reading.
+	 */
+	static FileStreamData* getStreamData(const char * fileName);
+
+	/** Destructor.
+	 *
+	 * Destroys CXref internal data and deallocates pdfWriter.  Note that
+	 * file handle provided to the constructor has to be closed by
+	 * descendants and it has to be closed after the instance is destroyed.
+	 * FileStreamDataDeleter is the preferred way to do that.
+	 */
+	virtual ~PdfDocumentWriter();
+
+	/** Sets new pdf content writer.
+	 * @param pdfWriter IPdfWriter interface implementator.
+	 *
+	 * If given parameter is not NULL, sets new value of pdfWriter field and
+	 * returns an old one. Otherwise just returns current one.
+	 * <br>
+	 * NOTE that caller is responsible for deallocation if provides new one.
+	 *
+	 * @return Currently set implementator or old value if parameter is non NULL.
+	 */
+	IPdfWriter * setPdfWriter(IPdfWriter * pdfWriter)
+	{
+		IPdfWriter *current=pdfWriter;
+		
+		if(pdfWriter)
+			this->pdfWriter=pdfWriter;
+
+		return current;
+	}
+	
+};
+
 } // namespace utils
 
 } // namespace pdfobjects
