@@ -1,94 +1,37 @@
-/*                                                                              
- * PDFedit - free program for PDF document manipulation.                        
- * Copyright (C) 2006, 2007  PDFedit team:      Michal Hocko, 
- *                                              Miroslav Jahoda,       
- *                                              Jozef Misutka, 
- *                                              Martin Petricek                                             
+/*
+ * PDFedit - free program for PDF document manipulation.
+ * Copyright (C) 2006-2009  PDFedit team: Michal Hocko,
+ *                                        Jozef Misutka,
+ *                                        Martin Petricek
+ *                   Former team members: Miroslav Jahoda
  *
- * Project is hosted on http://sourceforge.net/projects/pdfedit                                                                      
- */ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program (in doc/LICENSE.GPL); if not, write to the 
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
+ * MA  02111-1307  USA
+ *
+ * Project is hosted on http://sourceforge.net/projects/pdfedit
+ */
 #ifndef _OBJECTCOMPARATOR_H_
 #define _OBJECTCOMPARATOR_H_
 
 #include <map>
-#include "xpdf/Object.h"
 
-/*
- * $RCSfile: objectstorage.h,v $
- * 
- * $Log: objectstorage.h,v $
- * Revision 1.10  2007/08/25 14:43:47  mstsxfx
- * formating
- *
- * Revision 1.9  2007/04/25 18:24:36  bilboq
- *
- * applied patch from Martin Michlmayr to fix compilation with gcc-4.3
- *
- * fixed some typos in comments
- *
- * Revision 1.8  2007/02/04 20:17:03  mstsxfx
- * Common Licence comment for all cc and h files available in doc/licence_header
- * file and its content to all cc and h files in src/{gui,kernel,utils}
- * directories.
- * Xpdf code, QSA and QOutputDevice code are not touched
- *
- * Revision 1.7  2006/04/27 17:44:16  hockm0bm
- * minor documentation update
- *
- * Revision 1.6  2006/04/21 19:26:41  hockm0bm
- * RefComparator changed because it didn't have correct behaviour
- *
- * Revision 1.5  2006/03/23 22:32:53  hockm0bm
- * documentation updated
- *
- * Revision 1.4  2006/02/28 19:10:12  hockm0bm
- * Compile errors corrected
- *
- * Revision 1.3  2006/02/28 19:04:00  hockm0bm
- * Add iterators manipulation with ObjectStorage
- *
- * Revision 1.2  2006/02/13 19:05:35  hockm0bm
- * Template implementation of class.
- *
- * Revision 1.1  2006/01/29 17:09:07  hockm0bm
- * Object storage implementation
- * TODO - implement in template way
- *
- *
- */
-
-/** Reference comparator class.
- *
- * Implements functional operator to compare two referencies defined as
- * Ref structures.
- * <br>
- * This can be used as comparator predicate in ObjectStorage where value is
- * Object type.
- */
-class RefComparator
-{
-public:
-        /** Comparator operator.
-         * @param v1 First value.
-         * @param v2 Second value.
-         *
-         * @returns true iff v1.num and v1.gen are less than v2.num and v2.gen.
-         *
-         */ 
-        bool operator()(const Ref & v1, const Ref & v2)const
-        {
-                if(v1.num == v2.num)
-                        return v1.gen < v2.gen;
-                else
-                        return v1.num < v2.num;
-        }
-};
 
 /**
  * @file objectstorage.h
  *
  * File which implements template object storage class. This is basicaly
- * mapping keys to objects and provide simple interface to manipulate 
+ * mapping keys to objects and provide simple interface to manipulate
  * with it. It wrapps STL map class functionality.
  */
 
@@ -130,7 +73,7 @@ public:
          * and puts to the mapping.
          * Otherwise update value of the previous mapping.
          * <br>
-         * This method doesn't invalidate existing (if any) iterator pointing 
+         * This method doesn't invalidate existing (if any) iterator pointing
          * to the association.
          *
          * @returns Value of the previous mapping or 0 if the key was inserted
@@ -145,12 +88,12 @@ public:
                         mapping.insert(Association(key, value));
                         return 0;
                 }
-                
+
                 V old=iter->second;
-                
+
                 // sets new value for association
                 iter->second=value;
-                
+
                 // returns old value
                 return old;
         }
@@ -160,7 +103,7 @@ public:
          *
          * Gets value associated with the key.
          *
-         * @return value of the value (direct pointer) or 0 if no such key 
+         * @return value of the value (direct pointer) or 0 if no such key
          * found.
          */
         V get(K & key)const
@@ -207,7 +150,7 @@ public:
                 // removes association
                 V old=get(key);
                 mapping.erase(key);
-                
+
                 return old;
         }
 
@@ -228,7 +171,7 @@ public:
         {
                 return mapping.begin();
         }
-        
+
         /** Returns iterator to end iterator.
          *
          * @return Iterator instance.
