@@ -20,17 +20,16 @@
 #ifndef STREAM_H
 #define STREAM_H
 
-#include <aconf.h>
+#include <xpdf-aconf.h>
 
 #ifdef USE_GCC_PRAGMAS
 #pragma interface
 #endif
 
 #include <stdio.h>
-#include "gtypes.h"
-#include "Object.h"
+#include "goo/gtypes.h"
+#include "xpdf/Object.h"
 
-class Decrypt;
 class BaseStream;
 
 //------------------------------------------------------------------------
@@ -56,6 +55,15 @@ enum StreamColorSpaceMode {
   streamCSDeviceCMYK
 };
 
+//------------------------------------------------------------------------
+
+// This is in Stream.h instead of Decrypt.h to avoid really annoying
+// include file dependency loops.
+enum CryptAlgorithm {
+  cryptRC4,
+  cryptAES
+};
+
 //------------------------------------------------------------------------
 // Stream (base class)
 //------------------------------------------------------------------------
@@ -115,6 +123,10 @@ public:
   // Get the BaseStream of this stream.
   virtual BaseStream *getBaseStream() = 0;
 
+  // Get the stream after the last decoder (this may be a BaseStream
+  // or a DecryptStream).
+  virtual Stream *getUndecodedStream() = 0;
+
   // Get the dictionary associated with this stream.
   virtual Dict *getDict() = 0;
 
@@ -122,8 +134,11 @@ public:
   virtual GBool isEncoder() { return gFalse; }
 
   // Get image parameters which are defined by the stream contents.
-  virtual void getImageParams(__attribute__((unused))	int *bitsPerComponent,
-			      __attribute__((unused))	StreamColorSpaceMode *csMode) {}
+  virtual void getImageParams(UNUSED_PARAM int *bitsPerComponent,
+			      UNUSED_PARAM StreamColorSpaceMode *csMode) {}
+
+  // Return the next stream in the "stack".
+  virtual Stream *getNextStream() { return NULL; }
 
   // Add filters to this stream according to the parameters in <dict>.
   // Returns the new stream.
@@ -152,7 +167,9 @@ public:
   virtual void setPos(Guint pos, int dir = 0) = 0;
   virtual GBool isBinary(GBool last = gTrue) { return last; }
   virtual BaseStream *getBaseStream() { return this; }
+  virtual Stream *getUndecodedStream() { return this; }
   virtual Dict *getDict() { return dict.getDict(); }
+  virtual GString *getFileName() { return NULL; }
 
   virtual Stream * clone()=0;
 
@@ -160,13 +177,7 @@ public:
   virtual Guint getStart() = 0;
   virtual void moveStart(int delta) = 0;
 
-  // Set decryption for this stream.
-  virtual void doDecryption(Guchar *fileKey, int keyLength,
-			    int objNum, int objGen);
-
 protected:
-
-  Decrypt *decrypt;
   Object dict;
 };
 
@@ -186,7 +197,9 @@ public:
   virtual int getPos() { return str->getPos(); }
   virtual void setPos(Guint pos, int dir = 0);
   virtual BaseStream *getBaseStream() { return str->getBaseStream(); }
+  virtual Stream *getUndecodedStream() { return str->getUndecodedStream(); }
   virtual Dict *getDict() { return str->getDict(); }
+  virtual Stream *getNextStream() { return str; }
 
 protected:
 
@@ -258,6 +271,8 @@ public:
 
   ~StreamPredictor();
 
+  GBool isOk() { return ok; }
+
   int lookChar();
   int getChar();
 
@@ -275,6 +290,7 @@ private:
   int rowBytes;			// bytes per line
   Guchar *predLine;		// line buffer
   int predIdx;			// current index in predLine
+  GBool ok;
 
   // initialize internal structures from context variables
   void initContext(int predictorA, int widthA, int nCompsA, int nBitsA);             
@@ -291,6 +307,9 @@ public:
 
   FileStream(FILE *fA, Guint startA, GBool limitedA,
 	     Guint lengthA, Object *dictA);
+  // Note that file handle is not closed in constructor!!!
+  // This is because of makeSubStream which creates a new
+  // file stream with the shared file handled. 
   virtual ~FileStream();
   virtual Stream *makeSubStream(Guint startA, GBool limitedA,
 				Guint lengthA, Object *dictA);
@@ -346,8 +365,6 @@ public:
   virtual void setPos(Guint pos, int dir = 0);
   virtual Guint getStart() { return start; }
   virtual void moveStart(int delta);
-  virtual void doDecryption(Guchar *fileKey, int keyLength,
-			    int objNum, int objGen);
 
 protected:
   char *buf;
@@ -554,18 +571,20 @@ private:
   int row;			// current row
   int inputBuf;			// input buffer
   int inputBits;		// number of bits in input buffer
-  short *refLine;		// reference line changing elements
-  int b1;			// index into refLine
-  short *codingLine;		// coding line changing elements
-  int a0;			// index into codingLine
+  int *codingLine;		// coding line changing elements
+  int *refLine;			// reference line changing elements
+  int a0i;			// index into codingLine
+  GBool err;			// error on current line
   int outputBits;		// remaining ouput bits
   int buf;			// character buffer
 
+  void addPixels(int a1, int black);
+  void addPixelsNeg(int a1, int black);
   short getTwoDimCode();
   short getWhiteCode();
   short getBlackCode();
   short lookBits(int n);
-  void eatBits(int n) { inputBits -= n; }
+  void eatBits(int n) { if ((inputBits -= n) < 0) inputBits = 0; }
 };
 
 //------------------------------------------------------------------------
@@ -601,11 +620,12 @@ struct DCTHuffTable {
 class DCTStream: public FilterStream {
 public:
 
-  DCTStream(Stream *strA);
+  DCTStream(Stream *strA, int colorXformA);
   virtual ~DCTStream();
   virtual StreamKind getKind() { return strDCT; }
-  virtual void reset();
   virtual Stream * clone();
+  virtual void reset();
+  virtual void close();
   virtual int getChar();
   virtual int lookChar();
   virtual GString *getPSFilter(int psLevel, char *indent);
@@ -622,7 +642,9 @@ private:
   DCTCompInfo compInfo[4];	// info for each component
   DCTScanInfo scanInfo;		// info for the current scan
   int numComps;			// number of components in image
-  int colorXform;		// need YCbCr-to-RGB transform?
+  int colorXform;		// color transform: -1 = unspecified
+				//                   0 = none
+				//                   1 = YUV/YUVK -> RGB/CMYK
   GBool gotJFIFMarker;		// set if APP0 JFIF marker was present
   GBool gotAdobeMarker;		// set if APP14 Adobe marker was present
   int restartInterval;		// restart interval, in MCUs
@@ -765,8 +787,8 @@ public:
   virtual Stream * clone();
   virtual int getChar() { return EOF; }
   virtual int lookChar() { return EOF; }
-  virtual GString *getPSFilter(__attribute__((unused))	int psLevel, __attribute__((unused))	char *indent)  { return NULL; }
-  virtual GBool isBinary(__attribute__((unused))	GBool last = gTrue) { return gFalse; }
+  virtual GString *getPSFilter(UNUSED_PARAM	int psLevel, UNUSED_PARAM	char *indent)  { return NULL; }
+  virtual GBool isBinary(UNUSED_PARAM	GBool last = gTrue) { return gFalse; }
 };
 
 //------------------------------------------------------------------------
@@ -783,7 +805,7 @@ public:
   virtual int getChar();
   virtual Stream * clone();
   virtual int lookChar();
-  virtual GString *getPSFilter(__attribute__((unused))	int psLevel, __attribute__((unused))	char *indent) { return NULL; }
+  virtual GString *getPSFilter(UNUSED_PARAM	int psLevel, UNUSED_PARAM	char *indent) { return NULL; }
   virtual GBool isBinary(GBool last = gTrue);
   virtual GBool isEncoder() { return gTrue; }
 
@@ -809,8 +831,8 @@ public:
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
   virtual int lookChar()
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
-  virtual GString *getPSFilter(__attribute__((unused))	int psLevel, __attribute__((unused))	char *indent) { return NULL; }
-  virtual GBool isBinary(__attribute__((unused))	GBool last = gTrue) { return gFalse; }
+  virtual GString *getPSFilter(UNUSED_PARAM	int psLevel, UNUSED_PARAM	char *indent) { return NULL; }
+  virtual GBool isBinary(UNUSED_PARAM	GBool last = gTrue) { return gFalse; }
   virtual GBool isEncoder() { return gTrue; }
 
 private:
@@ -840,8 +862,8 @@ public:
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
   virtual int lookChar()
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
-  virtual GString *getPSFilter(__attribute__((unused))	int psLevel, __attribute__((unused))	char *indent) { return NULL; }
-  virtual GBool isBinary(__attribute__((unused))	GBool last = gTrue) { return gFalse; }
+  virtual GString *getPSFilter(UNUSED_PARAM	int psLevel, UNUSED_PARAM	char *indent) { return NULL; }
+  virtual GBool isBinary(UNUSED_PARAM	GBool last = gTrue) { return gFalse; }
   virtual GBool isEncoder() { return gTrue; }
 
 private:
@@ -871,8 +893,8 @@ public:
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
   virtual int lookChar()
     { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
-  virtual GString *getPSFilter(__attribute__((unused))	int psLevel, __attribute__((unused))	char *indent) { return NULL; }
-  virtual GBool isBinary(__attribute__((unused))	GBool last = gTrue) { return gTrue; }
+  virtual GString *getPSFilter(UNUSED_PARAM	int psLevel, UNUSED_PARAM	char *indent) { return NULL; }
+  virtual GBool isBinary(UNUSED_PARAM	GBool last = gTrue) { return gTrue; }
   virtual GBool isEncoder() { return gTrue; }
 
 private:
